package com.fastbee.data.controller.deviceOperationRecords;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.fastbee.common.annotation.Log;
import com.fastbee.common.core.controller.BaseController;
import com.fastbee.common.core.domain.AjaxResult;
import com.fastbee.common.enums.BusinessType;
import com.fastbee.iot.domain.NgDeviceOperationRecords;
import com.fastbee.iot.service.INgDeviceOperationRecordsService;
import com.fastbee.common.utils.poi.ExcelUtil;
import com.fastbee.common.core.page.TableDataInfo;

/**
 * 设备操作记录Controller
 *
 * @author kerwincui
 * @date 2024-12-30
 */
@RestController
@RequestMapping("/iot/records")
@Api(tags = "设备操作记录")
public class NgDeviceOperationRecordsController extends BaseController
{
    @Autowired
    private INgDeviceOperationRecordsService ngDeviceOperationRecordsService;

    /**
     * 查询设备操作记录列表
     */
    @GetMapping("/list")
    @ApiOperation("查询设备操作记录列表")
    public TableDataInfo list(NgDeviceOperationRecords ngDeviceOperationRecords)
    {
        startPage();
        List<NgDeviceOperationRecords> list = ngDeviceOperationRecordsService.selectNgDeviceOperationRecordsList(ngDeviceOperationRecords);
        return getDataTable(list);
    }

    /**
     * 导出设备操作记录列表
     */
    @ApiOperation("导出设备操作记录列表")
    @PostMapping("/export")
    public void export(HttpServletResponse response, NgDeviceOperationRecords ngDeviceOperationRecords)
    {
        List<NgDeviceOperationRecords> list = ngDeviceOperationRecordsService.selectNgDeviceOperationRecordsList(ngDeviceOperationRecords);
        ExcelUtil<NgDeviceOperationRecords> util = new ExcelUtil<NgDeviceOperationRecords>(NgDeviceOperationRecords.class);
        util.exportExcel(response, list, "设备操作记录数据");
    }

    /**
     * 获取设备操作记录详细信息
     */
    @GetMapping(value = "/{id}")
    @ApiOperation("获取设备操作记录详细信息")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(ngDeviceOperationRecordsService.selectNgDeviceOperationRecordsById(id));
    }

    /**
     * 新增设备操作记录
     */
    @PostMapping
    @ApiOperation("新增设备操作记录")
    public AjaxResult add(@RequestBody NgDeviceOperationRecords ngDeviceOperationRecords)
    {
        return toAjax(ngDeviceOperationRecordsService.insertNgDeviceOperationRecords(ngDeviceOperationRecords));
    }

    /**
     * 修改设备操作记录
     */
    @PutMapping
    @ApiOperation("修改设备操作记录")
    public AjaxResult edit(@RequestBody NgDeviceOperationRecords ngDeviceOperationRecords)
    {
        return toAjax(ngDeviceOperationRecordsService.updateNgDeviceOperationRecords(ngDeviceOperationRecords));
    }

    /**
     * 删除设备操作记录
     */
    @DeleteMapping("/{ids}")
    @ApiOperation("删除设备操作记录")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(ngDeviceOperationRecordsService.deleteNgDeviceOperationRecordsByIds(ids));
    }
}
