package com.fastbee.data.controller.userRecharge;

import java.util.Calendar;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import javax.servlet.http.HttpServletResponse;

import cn.hutool.core.date.DateTime;
import com.fastbee.rechargecard.domain.dto.NgUserRechargeRecordsDto;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.fastbee.common.annotation.Log;
import com.fastbee.common.core.controller.BaseController;
import com.fastbee.common.core.domain.AjaxResult;
import com.fastbee.common.enums.BusinessType;
import com.fastbee.rechargecard.domain.NgUserRechargeRecords;
import com.fastbee.rechargecard.service.INgUserRechargeRecordsService;
import com.fastbee.common.utils.poi.ExcelUtil;
import com.fastbee.common.core.page.TableDataInfo;

/**
 * 用户充值记录Controller
 *
 * @author kerwincui
 * @date 2024-12-19
 */
@RestController
@RequestMapping("/rechargecard/records")
@Api(tags = "用户充值记录")
public class NgUserRechargeRecordsController extends BaseController
{
    @Autowired
    private INgUserRechargeRecordsService ngUserRechargeRecordsService;

    /**
     * 统计充值记录中的充值总金额和充值用户数量
     * @return
     */
    @GetMapping("/total")
    @ApiOperation("统计充值记录中的充值总金额和充值用户数量")
    public AjaxResult total(NgUserRechargeRecordsDto ngUserRechargeRecordsDto)
    {
        return success(ngUserRechargeRecordsService.getTotal(ngUserRechargeRecordsDto));
    }

    /**
     * 查询用户充值记录列表
     */
//    @PreAuthorize("@ss.hasPermi('rechargecard:records:list')")
    @GetMapping("/list")
    @ApiOperation("查询用户充值记录列表")
    public TableDataInfo list(NgUserRechargeRecordsDto ngUserRechargeRecords)
    {
        startPage();
        if(ngUserRechargeRecords.getStartTime()!=null && ngUserRechargeRecords.getEndTime()!=null)
        {
            //设置结束时间加一天
            Date endTime = ngUserRechargeRecords.getEndTime();
            // 使用 Calendar 给日期加一天
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(endTime);
            calendar.add(Calendar.DATE, 1); // 给日期加一天
            // 将计算后的日期设置回 userIrrigationRecord 对象
            ngUserRechargeRecords.setEndTime(calendar.getTime());
        }
        List<NgUserRechargeRecords> list = ngUserRechargeRecordsService.selectNgUserRechargeRecordsListByTime(ngUserRechargeRecords);
        //按照充值时间由近到远排序
        // list.sort(Comparator.comparing(NgUserRechargeRecords::getRechargeTime).reversed());
        // System.err.println("充值list = " + list);
        return getDataTable(list);
    }



    /**
     * 导出用户充值记录列表
     */
    @ApiOperation("导出用户充值记录列表")
//    @PreAuthorize("@ss.hasPermi('rechargecard:records:export')")
    @PostMapping("/export")
    public void export(HttpServletResponse response, NgUserRechargeRecordsDto ngUserRechargeRecords)
    {
        if(ngUserRechargeRecords.getStartTime()!=null && ngUserRechargeRecords.getEndTime()!=null)
        {
            //设置结束时间加一天
            Date endTime = ngUserRechargeRecords.getEndTime();
            // 使用 Calendar 给日期加一天
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(endTime);
            calendar.add(Calendar.DATE, 1); // 给日期加一天
            // 将计算后的日期设置回 userIrrigationRecord 对象
            ngUserRechargeRecords.setEndTime(calendar.getTime());
        }

//        List<NgUserRechargeRecords> list = ngUserRechargeRecordsService.selectNgUserRechargeRecordsList(ngUserRechargeRecords);
        List<NgUserRechargeRecords> list = ngUserRechargeRecordsService.selectNgUserRechargeRecordsListByTime(ngUserRechargeRecords);
        ExcelUtil<NgUserRechargeRecords> util = new ExcelUtil<NgUserRechargeRecords>(NgUserRechargeRecords.class);
        util.exportExcel(response, list, "用户充值记录数据");
    }

    /**
     * 获取用户充值记录详细信息
     */
//    @PreAuthorize("@ss.hasPermi('rechargecard:records:query')")
    @GetMapping(value = "/user")
    @ApiOperation("获取用户充值记录详细信息")
    public TableDataInfo getInfo(NgUserRechargeRecords ngUserRechargeRecords)
    {
        startPage();
        List<NgUserRechargeRecords> list = ngUserRechargeRecordsService.selectNgUserRechargeRecordsByCardNumber(ngUserRechargeRecords);
        return getDataTable(list);
    }

    /**
     * 获取充值机充值记录详细信息
     */
//    @PreAuthorize("@ss.hasPermi('rechargecard:records:query')")
    /*@GetMapping(value = "/serial/{serialNumber}")*/
    @GetMapping(value = "/serial")
    @ApiOperation("获取充值机充值记录详细信息")
    public TableDataInfo getSerialInfo(NgUserRechargeRecords ngUserRechargeRecords)
    {
        startPage();
        List<NgUserRechargeRecords> list = ngUserRechargeRecordsService.selectNgUserRechargeRecordsBySerialNumber(ngUserRechargeRecords);
        return getDataTable(list);
    }



    /**
     * 新增用户充值记录
     */
//    @PreAuthorize("@ss.hasPermi('rechargecard:records:add')")
    @PostMapping
    @ApiOperation("新增用户充值记录")
    public AjaxResult add(@RequestBody NgUserRechargeRecords ngUserRechargeRecords)
    {
        return toAjax(ngUserRechargeRecordsService.insertNgUserRechargeRecords(ngUserRechargeRecords));
    }

    /**
     * 修改用户充值记录
     */
//    @PreAuthorize("@ss.hasPermi('rechargecard:records:edit')")
    @PutMapping
    @ApiOperation("修改用户充值记录")
    public AjaxResult edit(@RequestBody NgUserRechargeRecords ngUserRechargeRecords)
    {
        return toAjax(ngUserRechargeRecordsService.updateNgUserRechargeRecords(ngUserRechargeRecords));
    }

    /**
     * 删除用户充值记录
     */
//    @PreAuthorize("@ss.hasPermi('rechargecard:records:remove')")
    @DeleteMapping("/{ids}")
    @ApiOperation("删除用户充值记录")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(ngUserRechargeRecordsService.deleteNgUserRechargeRecordsByIds(ids));
    }
}
