package com.fastbee.data.controller.userRecharge;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.*;
import javax.servlet.http.HttpServletResponse;

import com.fastbee.common.core.domain.entity.SysUser;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.fastbee.rechargecard.domain.UserConsumptionDetails;
import com.fastbee.rechargecard.domain.dto.NgUserRechargeRecordsDto;
import com.fastbee.rechargecard.domain.dto.UserIrrigationRecordDto;
import com.fastbee.rechargecard.domain.dto.UserIrrigationRecordListDto;
import com.fastbee.rechargecard.mapper.UserIrrigationRecordMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.data.repository.query.Param;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.fastbee.common.annotation.Log;
import com.fastbee.common.core.controller.BaseController;
import com.fastbee.common.core.domain.AjaxResult;
import com.fastbee.common.enums.BusinessType;
import com.fastbee.rechargecard.domain.UserIrrigationRecord;
import com.fastbee.rechargecard.service.IUserIrrigationRecordService;
import com.fastbee.common.utils.poi.ExcelUtil;
import com.fastbee.common.core.page.TableDataInfo;

/**
 * 灌溉记录Controller
 *
 * @author kerwincui
 * @date 2024-12-18
 */
@RestController
@RequestMapping("/rechargecard/record")
@Api(tags = "灌溉记录")
public class UserIrrigationRecordController extends BaseController
{
    @Autowired
    private IUserIrrigationRecordService userIrrigationRecordService;

    @Autowired
    private UserIrrigationRecordMapper userIrrigationRecordMapper;

    /**
     * 统计灌溉记录中的总用水量和总用水户
     * @return
     */
    @GetMapping("/total")
    @ApiOperation("统计灌溉记录中的总用水量和总用水户")
    public AjaxResult total(UserIrrigationRecordListDto userIrrigationRecordListDto)
    {
        //return success(ngUserRechargeRecordsService.getTotal(ngUserRechargeRecordsDto));
        return success(userIrrigationRecordService.getTotal(userIrrigationRecordListDto));
    }

    /**
     * 查询灌溉记录列表
     */
//    @PreAuthorize("@ss.hasPermi('rechargecard:record:list')")
    @GetMapping("/list")
    @ApiOperation("查询灌溉记录列表")
    public TableDataInfo list(UserIrrigationRecordListDto userIrrigationRecord)
    {
        startPage();
        if(userIrrigationRecord.getSearchEndTime()!=null && userIrrigationRecord.getSearchStartTime()!=null)
        {
            //设置结束时间加一天
            Date searchEndTime = userIrrigationRecord.getSearchEndTime();
            // 使用 Calendar 给日期加一天
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(searchEndTime);
            calendar.add(Calendar.DATE, 1); // 给日期加一天
            // 将计算后的日期设置回 userIrrigationRecord 对象
            userIrrigationRecord.setSearchEndTime(calendar.getTime());
        }

        List<UserIrrigationRecordListDto> list = userIrrigationRecordService.selectUserIrrigationRecordListByTime(userIrrigationRecord);
        return getDataTable(list);
    }

    /**
     * 获取用户灌溉记录详细信息列表
     */
//    @PreAuthorize("@ss.hasPermi('rechargecard:record:query')")
    @GetMapping(value = "/list/{cardNumber}")
    @ApiOperation("获取灌溉记录详细信息")
    public AjaxResult getInfo(@PathVariable("cardNumber") String cardNumber)
    {
        return success(userIrrigationRecordService.selectUserIrrigationRecordListBycardNumber(cardNumber));
    }

    /**
     * 获取全部灌溉记录展示详细信息列表
     */
//    @PreAuthorize("@ss.hasPermi('rechargecard:record:list')")
    @GetMapping("/list/show")
    @ApiOperation("查询灌溉记录列表")
    public TableDataInfo Showlist(UserIrrigationRecord userIrrigationRecord)
    {
        startPage();
        List<UserIrrigationRecordDto> list = userIrrigationRecordService.selectUserIrrigationRecordShowList(userIrrigationRecord);
        return getDataTable(list);
    }

    /**
     * 获取用户灌溉记录展示详细信息列表
     */
//    @PreAuthorize("@ss.hasPermi('rechargecard:record:list')")
    @GetMapping("/list/show/user")
    @ApiOperation("查询用户灌溉记录列表")
    public TableDataInfo ShowlistByCardNumber(UserIrrigationRecord userIrrigationRecord)
    {
        startPage();
        List<UserIrrigationRecordDto> list = userIrrigationRecordService.selectUserIrrigationRecordShowListByCardNumber(userIrrigationRecord);
        return getDataTable(list);
    }

    /**
     * 获取灌溉控制器灌溉记录展示详细信息列表
     */
//    @PreAuthorize("@ss.hasPermi('rechargecard:record:list')")
    @GetMapping("/list/device/{deviceNumber}")
    @ApiOperation("查询灌溉控制器灌溉记录列表")
    public TableDataInfo ShowlistByDeviceNumber(@PathVariable("deviceNumber") String deviceNumber)
    {
        startPage();
        List<UserIrrigationRecord> list = userIrrigationRecordService.selectUserIrrigationRecordShowListByDeviceNumber(deviceNumber);
        return getDataTable(list);
    }


    /**
     * 导出灌溉记录列表
     */
    @ApiOperation("导出灌溉记录列表")
//    @PreAuthorize("@ss.hasPermi('rechargecard:record:export')")
    @PostMapping("/export")
    public void export(HttpServletResponse response, UserIrrigationRecordListDto userIrrigationRecord)
    {
        if(userIrrigationRecord.getSearchEndTime()!=null && userIrrigationRecord.getSearchStartTime()!=null)
        {
            //设置结束时间加一天
            Date searchEndTime = userIrrigationRecord.getSearchEndTime();
            // 使用 Calendar 给日期加一天
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(searchEndTime);
            calendar.add(Calendar.DATE, 1); // 给日期加一天
            // 将计算后的日期设置回 userIrrigationRecord 对象
            userIrrigationRecord.setSearchEndTime(calendar.getTime());
        }

        List<UserIrrigationRecordListDto> list = userIrrigationRecordService.selectUserIrrigationRecordListByTime(userIrrigationRecord);
        //List<UserIrrigationRecord> list = userIrrigationRecordService.selectUserIrrigationRecordList(userIrrigationRecord);
        ExcelUtil<UserIrrigationRecordListDto> util = new ExcelUtil<UserIrrigationRecordListDto>(UserIrrigationRecordListDto.class);
        util.exportExcel(response, list, "灌溉记录数据");
    }

    /**
     * 获取灌溉记录详细信息
     */
//    @PreAuthorize("@ss.hasPermi('rechargecard:record:query')")
    @GetMapping(value = "/{id}")
    @ApiOperation("获取灌溉记录详细信息")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(userIrrigationRecordService.selectUserIrrigationRecordById(id));
    }

    /**
     * 新增灌溉记录
     */
//    @PreAuthorize("@ss.hasPermi('rechargecard:record:add')")
    @PostMapping
    @ApiOperation("新增灌溉记录")
    public AjaxResult add(@RequestBody UserIrrigationRecord userIrrigationRecord)
    {
        return toAjax(userIrrigationRecordService.insertUserIrrigationRecord(userIrrigationRecord));
    }

    /**
     * 修改灌溉记录
     */
//    @PreAuthorize("@ss.hasPermi('rechargecard:record:edit')")
    @PutMapping
    @ApiOperation("修改灌溉记录")
    public AjaxResult edit(@RequestBody UserIrrigationRecord userIrrigationRecord)
    {
        return toAjax(userIrrigationRecordService.updateUserIrrigationRecord(userIrrigationRecord));
    }

    /**
     * 删除灌溉记录
     */
//    @PreAuthorize("@ss.hasPermi('rechargecard:record:remove')")
    @DeleteMapping("/{ids}")
    @ApiOperation("删除灌溉记录")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(userIrrigationRecordService.deleteUserIrrigationRecordByIds(ids));
    }



    @GetMapping("/getYearWater")
    @ApiOperation("统计用户的年度用水量，年度剩余用水量")
    public AjaxResult getYearWater(@Param("userId") Long userId) {
        // 获取当前年份的第一天
        LocalDate startOfYear = LocalDate.now().withDayOfYear(1);

        // 获取从今年第一天开始以后的所有用水量记录
        List<UserIrrigationRecord> records = new LambdaQueryChainWrapper<>(userIrrigationRecordMapper)
                .select(UserIrrigationRecord::getCurFlow)
                .eq(UserIrrigationRecord::getUserId, userId)
                .ge(UserIrrigationRecord::getCreateTime, startOfYear)
                .list();

        // 手动累加CurFlow字段的值，CurFlow字段是BigDecimal类型
        BigDecimal totalWaterUsed = records.stream()
                .map(UserIrrigationRecord::getCurFlow) // 将记录转换为BigDecimal流
                .reduce(BigDecimal.ZERO, BigDecimal::add); // 使用reduce累加BigDecimal值

        Map<String,Object> resp = new HashMap<>();
        resp.put("yearUserWater", totalWaterUsed);
        resp.put("yearUserWaterLeft", BigDecimal.valueOf(1000).subtract(totalWaterUsed)); // 假设年度总用水量为1000单位
        return AjaxResult.success(resp);
    }

    public static void main(String[] args) {
        System.err.println(LocalDate.now().withDayOfYear(1));
    }
}
