package com.fastbee.ggroup.service;

import java.util.List;
import com.fastbee.ggroup.domain.GSiteDevice;
import com.fastbee.ggroup.domain.dto.GSiteDeviceDto;
import com.fastbee.ggroup.domain.vo.GSiteDeviceVo;
import com.fastbee.iot.domain.Device;

/**
 * 站点设备关系Service接口
 * 
 * @author kerwincui
 * @date 2024-11-05
 */

public interface IGSiteDeviceService 
{

    /**
     * 查询站点未关联的设备列表
     */
    public List<GSiteDeviceVo> selectUnbindDeviceList(Long siteId,Long groupId);

    /**
     * 查询站点已关联设备列表
     * @param siteId 站点设备关系主键
     */
    public List<GSiteDeviceVo> selectBindDeviceList(Long siteId,Long groupId);


    /**
     * 查询站点设备关系
     * 
     * @param id 站点设备关系主键
     * @return 站点设备关系
     */
    public GSiteDevice selectGSiteDeviceById(Long id);

    /**
     * 查询站点设备关系列表
     * 
     * @param gSiteDevice 站点设备关系
     * @return 站点设备关系集合
     */
    public List<GSiteDevice> selectGSiteDeviceList(GSiteDevice gSiteDevice);

    /**
     * 新增站点设备关系
     * 
     * @param gSiteDevice 站点设备关系
     * @return 结果
     */
    public int insertGSiteDevice(GSiteDevice gSiteDevice);

    /**
     * 修改站点设备关系
     * 
     * @param gSiteDevice 站点设备关系
     * @return 结果
     */
    public int updateGSiteDevice(GSiteDevice gSiteDevice);

    /**
     * 批量删除站点设备关系
     * 
     * @param ids 需要删除的站点设备关系主键集合
     * @return 结果
     */
    public int deleteGSiteDeviceByIds(Long[] ids);

    /**
     * 删除站点设备关系信息
     * 
     * @param id 站点设备关系主键
     * @return 结果
     */
    public int deleteGSiteDeviceById(Long id);



}
