package com.fastbee.iot.service.impl;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.*;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fastbee.deviceData.domain.DeviceRealtimedataMeteorology;
import com.fastbee.deviceData.mapper.DeviceRealtimedataMeteorologyMapper;
import com.fastbee.iot.domain.dto.NgFlowDeviceDataDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.fastbee.iot.mapper.NgFlowDeviceDataMapper;
import com.fastbee.iot.domain.NgFlowDeviceData;
import com.fastbee.iot.service.INgFlowDeviceDataService;

/**
 * 流量计设备数据Service业务层处理
 * 
 * @author kerwincui
 * @date 2024-12-30
 */
@Service
public class NgFlowDeviceDataServiceImpl extends ServiceImpl<NgFlowDeviceDataMapper, NgFlowDeviceData> implements INgFlowDeviceDataService
{
    @Autowired
    private NgFlowDeviceDataMapper ngFlowDeviceDataMapper;

    /**
     * 获取流量计瞬时流量、累计流量图表
     * @param deviceNumber
     * @param startTime
     * @param endTime
     * @return
     */
    @Override
    public Map<String, Object> selectNgFlowDeviceDataFlowCharts(String deviceNumber, Date startTime, Date endTime) {
        QueryWrapper<NgFlowDeviceData> queryWrapper=new QueryWrapper<>();
        Map<String,Object> resultMap=new HashMap<>();
        List<BigDecimal> insList=new ArrayList<>();
        List<BigDecimal> sumList=new ArrayList<>();
        List<String> timeList=new ArrayList<>();

        Date start = DateUtil.beginOfDay(DateUtil.date());// 获取今日零点时间
        Date now = DateUtil.date();// 获取当前时间
        // 获取当前整点时刻的时间
        Date end = DateUtil.beginOfHour(now);//用当前整点时刻时间作为结束时间
        if (startTime != null && endTime != null) {
            //如果有时间筛选，按照时间筛选为起止时间
            start=startTime;
            end=endTime;
        }

        queryWrapper.between("real_time", start, end);
        queryWrapper.eq("device_number",deviceNumber);
        List<NgFlowDeviceData>  list=ngFlowDeviceDataMapper.selectList(queryWrapper);//获取根据时间和设备编码筛选后的数据

        while(start.compareTo(end) <=0 )//start<=end
        {
            //整点时间数据=前半个小时到后半个小时的平均数据，即12点数据=11:30到12：30的平均值
            Date tempStart=DateUtil.offsetMinute(start,-30);//前半个小时时间
            Date tempEnd=DateUtil.offsetMinute(start,30);//后半个小时时间
            if(tempEnd.after(now))
            {
                tempEnd=now;//用来处理当前是12：10但需要获取12点数据的情况
            }
            BigDecimal insData=new BigDecimal("0");//瞬时流量总和
            BigDecimal sumData=new BigDecimal("0");//累计流量总和
            int num=0;
            for(int i=0;i<list.size();i++)
            {
                NgFlowDeviceData temp=list.get(i);
                //如果时间符合条件
                if(temp.getRealTime().compareTo(tempStart)>=0 && temp.getRealTime().compareTo(tempEnd)<=0)
                {
                    BigDecimal insFlow=new BigDecimal(temp.getInFlow());//瞬时流量
                    insData=insData.add(insFlow);
                    BigDecimal sumFlow=new BigDecimal(temp.getSumFlow());//累计流量
                    sumData=sumData.add(sumFlow);
                    num++;
                }
            }
            BigDecimal insResult=new BigDecimal("0");
            BigDecimal sumResult=new BigDecimal("0");
            if(num>0)
            {
                insResult = insData.divide(new BigDecimal(String.valueOf(num)), 2, RoundingMode.HALF_UP);//计算瞬时流量平均值
                sumResult=sumData.divide(new BigDecimal(String.valueOf(num)),2,RoundingMode.HALF_UP);//计算累计流量平均值
            }
            SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");//格式化为时：分
            String formattedDate = sdf.format(start);
            timeList.add(formattedDate);
            insList.add(insResult);
            sumList.add(sumResult);

            start=DateUtil.offsetHour(start,1);//start添加一个小时
        }
        resultMap.put("time",timeList);
        resultMap.put("insFlow",insList);
        resultMap.put("sumFlow",sumList);

        return resultMap;
    }

    /**
     * 根据时间查询流量计历史数据
     * @param ngFlowDeviceData 流量计设备数据
     * @return
     */
    @Override
    public List<NgFlowDeviceData> selectNgFlowDeviceDataListByTime(NgFlowDeviceDataDto ngFlowDeviceData) {
        if(ngFlowDeviceData.getStartTime() != null && ngFlowDeviceData.getEndTime() != null)
        {
            //设置结束时间加一天
            Date searchEndTime = ngFlowDeviceData.getEndTime();
            // 使用 Calendar 给日期加一天
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(searchEndTime);
            calendar.add(Calendar.DATE, 1); // 给日期加一天
            // 将计算后的日期设置回 userIrrigationRecord 对象
            ngFlowDeviceData.setEndTime(calendar.getTime());
        }

        return ngFlowDeviceDataMapper.selectNgFlowDeviceDataListByTime(ngFlowDeviceData);
    }

    /**
     * 查询流量计设备数据
     * 
     * @param id 流量计设备数据主键
     * @return 流量计设备数据
     */
    @Override
    public NgFlowDeviceData selectNgFlowDeviceDataById(Long id)
    {
        return ngFlowDeviceDataMapper.selectNgFlowDeviceDataById(id);
    }

    /**
     * 查询流量计设备数据列表
     * 
     * @param ngFlowDeviceData 流量计设备数据
     * @return 流量计设备数据
     */
    @Override
    public List<NgFlowDeviceData> selectNgFlowDeviceDataList(NgFlowDeviceData ngFlowDeviceData)
    {
        return ngFlowDeviceDataMapper.selectNgFlowDeviceDataList(ngFlowDeviceData);
    }

    /**
     * 新增流量计设备数据
     * 
     * @param ngFlowDeviceData 流量计设备数据
     * @return 结果
     */
    @Override
    public int insertNgFlowDeviceData(NgFlowDeviceData ngFlowDeviceData)
    {
        return ngFlowDeviceDataMapper.insertNgFlowDeviceData(ngFlowDeviceData);
    }

    /**
     * 修改流量计设备数据
     * 
     * @param ngFlowDeviceData 流量计设备数据
     * @return 结果
     */
    @Override
    public int updateNgFlowDeviceData(NgFlowDeviceData ngFlowDeviceData)
    {
        return ngFlowDeviceDataMapper.updateNgFlowDeviceData(ngFlowDeviceData);
    }

    /**
     * 批量删除流量计设备数据
     * 
     * @param ids 需要删除的流量计设备数据主键
     * @return 结果
     */
    @Override
    public int deleteNgFlowDeviceDataByIds(Long[] ids)
    {
        return ngFlowDeviceDataMapper.deleteNgFlowDeviceDataByIds(ids);
    }

    /**
     * 删除流量计设备数据信息
     * 
     * @param id 流量计设备数据主键
     * @return 结果
     */
    @Override
    public int deleteNgFlowDeviceDataById(Long id)
    {
        return ngFlowDeviceDataMapper.deleteNgFlowDeviceDataById(id);
    }
}
