package com.fastbee.rechargecard.mapper;

import java.util.List;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.fastbee.rechargecard.domain.NgWaterPumpUsageRecords;
import com.fastbee.rechargecard.domain.dto.NgWaterPumpUsageRecordsDto;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Select;

/**
 * 水泵设备使用记录Mapper接口
 *
 * @author kerwincui
 * @date 2024-12-19
 */
@Mapper
public interface NgWaterPumpUsageRecordsMapper extends BaseMapper<NgWaterPumpUsageRecords>
{
    /**
     * 查询水泵设备使用记录列表
     *
     * @param ngWaterPumpUsageRecords 水泵设备使用记录
     * @return 水泵设备使用记录集合
     */
    public List<NgWaterPumpUsageRecords> selectNgWaterPumpUsageRecordsListByTime(NgWaterPumpUsageRecordsDto ngWaterPumpUsageRecords);
    /**
     * 查询水泵设备使用记录
     *
     * @param id 水泵设备使用记录主键
     * @return 水泵设备使用记录
     */
    public NgWaterPumpUsageRecords selectNgWaterPumpUsageRecordsById(Long id);

    /**
     * 查询水泵设备最细实时数据
     * @param deviceNumber 水泵设备编码
     * @return 水泵设备使用记录
     */
    @Select("SELECT * FROM ng_water_pump_usage_records WHERE" +
            " device_number=#{deviceNumber} ORDER BY pump_start_time DESC LIMIT 1")
    public NgWaterPumpUsageRecords selectNgWaterPumpUsageDeviceNumber(String deviceNumber);
    /**
     * 查询水泵设备使用记录列表
     *
     * @param ngWaterPumpUsageRecords 水泵设备使用记录
     * @return 水泵设备使用记录集合
     */
    public List<NgWaterPumpUsageRecords> selectNgWaterPumpUsageRecordsList(NgWaterPumpUsageRecords ngWaterPumpUsageRecords);

    /**
     * 新增水泵设备使用记录
     *
     * @param ngWaterPumpUsageRecords 水泵设备使用记录
     * @return 结果
     */
    public int insertNgWaterPumpUsageRecords(NgWaterPumpUsageRecords ngWaterPumpUsageRecords);

    /**
     * 修改水泵设备使用记录
     *
     * @param ngWaterPumpUsageRecords 水泵设备使用记录
     * @return 结果
     */
    public int updateNgWaterPumpUsageRecords(NgWaterPumpUsageRecords ngWaterPumpUsageRecords);

    /**
     * 删除水泵设备使用记录
     *
     * @param id 水泵设备使用记录主键
     * @return 结果
     */
    public int deleteNgWaterPumpUsageRecordsById(Long id);

    /**
     * 批量删除水泵设备使用记录
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteNgWaterPumpUsageRecordsByIds(Long[] ids);
}
