package com.fastbee.rechargecard.mapper;

import java.util.List;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.fastbee.rechargecard.domain.UserConsumptionDetails;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Select;

/**
 * 用户充值卡账单明细记录Mapper接口
 *
 * @author kerwincui
 * @date 2024-12-18
 */
@Mapper
public interface UserConsumptionDetailsMapper extends BaseMapper<UserConsumptionDetails>
{
    /**
     * 根据订单编号获取订单信息
     * @param orderNumber
     * @return
     */
    @Select("select * from user_consumption_details where order_number=#{orderNumber}")
    public UserConsumptionDetails selectUserConsumptionDetailsByOrderNumber(String orderNumber);

    /**
     * 根据卡号查询用户充值卡账单明细记录列表
     * @param cardNumber
     * @return
     */
    public List<UserConsumptionDetails> selectUserConsumptionDetailsListByCardNumber(String cardNumber);

    /**
     * 根据卡号查询用户充值卡账单明细记录列表，判断是否为刷卡记录，billingType=1为刷卡
     * @param
     * @return
     */
    public List<UserConsumptionDetails> selectUserConsumptionDetailsSwipListByCardNumber(UserConsumptionDetails userConsumptionDetails);
    /**
     * 查询用户充值卡账单明细记录
     * @param id
     * @return
     */
    public UserConsumptionDetails selectUserConsumptionDetailsById(Long id);

    /**
     * 查询用户充值卡账单明细记录列表
     *
     * @param userConsumptionDetails 用户充值卡账单明细记录
     * @return 用户充值卡账单明细记录集合
     */
    public List<UserConsumptionDetails> selectUserConsumptionDetailsList(UserConsumptionDetails userConsumptionDetails);

    /**
     * 新增用户充值卡账单明细记录
     *
     * @param userConsumptionDetails 用户充值卡账单明细记录
     * @return 结果
     */
    public int insertUserConsumptionDetails(UserConsumptionDetails userConsumptionDetails);

    /**
     * 修改用户充值卡账单明细记录
     *
     * @param userConsumptionDetails 用户充值卡账单明细记录
     * @return 结果
     */
    public int updateUserConsumptionDetails(UserConsumptionDetails userConsumptionDetails);

    /**
     * 删除用户充值卡账单明细记录
     *
     * @param id 用户充值卡账单明细记录主键
     * @return 结果
     */
    public int deleteUserConsumptionDetailsById(Long id);

    /**
     * 批量删除用户充值卡账单明细记录
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteUserConsumptionDetailsByIds(Long[] ids);
}
