package com.fastbee.rechargecard.mapper;

import java.util.List;
import com.fastbee.rechargecard.domain.UserRechargeCards;
import com.github.yulichang.base.MPJBaseMapper;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.springframework.stereotype.Repository;

/**
 * 用户充值卡Mapper接口
 *
 * @author kerwincui
 * @date 2024-12-17
 */
@Mapper
public interface UserRechargeCardsMapper extends MPJBaseMapper<UserRechargeCards>
{

    /**
     * 测试方法
     * @param id
     * @return
     */
    public Long test(Long id);

    /**
     * 根据卡号和区域号查询
     * @param cardNumber
     * @param areaCode
     * @return
     */
    @Select("select * from user_recharge_cards where card_number=#{cardNumber} and area_code=#{areaCode}")
    public UserRechargeCards selectUserRechargeCardsByCardNumberAndAreaCode(
            @Param("cardNumber") String cardNumber,
            @Param("areaCode") String areaCode
    );
    /**
     * 查询用户充值卡
     *
     * @param id 用户充值卡主键
     * @return 用户充值卡
     */
    public UserRechargeCards selectUserRechargeCardsById(Long id);
    /**
     * 查询用户充值卡
     *
     * @param cardNumber
     * @return 用户充值卡
     */
    @Select("select * from user_recharge_cards where card_number=#{cardNumber}")
    public UserRechargeCards selectUserRechargeCardsByCardNumber(String cardNumber);

    /**
     * card_number查询用户充值卡

     * @return 用户充值卡
     */
    public UserRechargeCards selectUserRechargeCardsByCardnumber(String cardnumber);
    /**
     * 查询用户充值卡列表
     *
     * @param userRechargeCards 用户充值卡
     * @return 用户充值卡集合
     */
    public List<UserRechargeCards> selectUserRechargeCardsList(UserRechargeCards userRechargeCards);

    /**
     * 新增用户充值卡
     *
     * @param userRechargeCards 用户充值卡
     * @return 结果
     */
    public int insertUserRechargeCards(UserRechargeCards userRechargeCards);

    /**
     * 修改用户充值卡
     *
     * @param userRechargeCards 用户充值卡
     * @return 结果
     */
    public int updateUserRechargeCards(UserRechargeCards userRechargeCards);

    /**
     * 删除用户充值卡
     *
     * @param id 用户充值卡主键
     * @return 结果
     */
    public int deleteUserRechargeCardsById(Long id);

    /**
     * 批量删除用户充值卡
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteUserRechargeCardsByIds(Long[] ids);
}
