package com.fastbee.rechargecard.service;

import java.util.List;
import java.util.Map;
import java.util.Objects;

import com.fastbee.rechargecard.domain.NgUserRechargeRecords;
import com.fastbee.rechargecard.domain.dto.NgUserRechargeRecordsDto;
import com.fastbee.rechargecard.domain.dto.RechargecardUser;
import com.fastbee.rechargecard.domain.dto.WeChatRecharge;
import com.fastbee.rechargecard.mapper.NgUserRechargeRecordsMapper;

/**
 * 用户充值记录Service接口
 *
 * @author kerwincui
 * @date 2024-12-19
 */
public interface INgUserRechargeRecordsService
{
    /**
     * 统计总充值金额和总充值用户
     * @return
     */
    public Map<String, Object> getTotal(NgUserRechargeRecordsDto ngUserRechargeRecordsDto);

    /**
     * 查询用户充值记录列表
     *
     * @param ngUserRechargeRecords 用户充值记录
     * @return 用户充值记录集合
     */
    public List<NgUserRechargeRecords> selectNgUserRechargeRecordsListByTime(NgUserRechargeRecordsDto ngUserRechargeRecords);

    /**
     * 根据订单编号查找充值记录
     * @param rechargeCode
     * @return
     */
    public NgUserRechargeRecords SelectRechargeRecodeByRechargeCode(String rechargeCode);

    /**
     * 微信充值-新增用户充值记录
     *
     * @param
     * @return 结果
     */
    public int insertNgUserRechargeRecordsWeChat(WeChatRecharge recharge);

    /**
     * 查询用户充值记录
     *
     * @param
     * @return 用户充值记录
     */
    public List<NgUserRechargeRecords> selectNgUserRechargeRecordsByCardNumber(NgUserRechargeRecords ngUserRechargeRecords);

    /**
     * 查询充值机充值记录
     *
     * @param
     * @return 用户充值记录
     */
    public List<NgUserRechargeRecords> selectNgUserRechargeRecordsBySerialNumber(NgUserRechargeRecords ngUserRechargeRecords);

    /**
     * 查询用户充值记录列表
     *
     * @param ngUserRechargeRecords 用户充值记录
     * @return 用户充值记录集合
     */
    public List<NgUserRechargeRecords> selectNgUserRechargeRecordsList(NgUserRechargeRecords ngUserRechargeRecords);

    /**
     * 新增用户充值记录
     *
     * @param ngUserRechargeRecords 用户充值记录
     * @return 结果
     */
    public int insertNgUserRechargeRecords(NgUserRechargeRecords ngUserRechargeRecords);

    /**
     * 修改用户充值记录
     *
     * @param ngUserRechargeRecords 用户充值记录
     * @return 结果
     */
    public int updateNgUserRechargeRecords(NgUserRechargeRecords ngUserRechargeRecords);

    /**
     * 批量删除用户充值记录
     *
     * @param ids 需要删除的用户充值记录主键集合
     * @return 结果
     */
    public int deleteNgUserRechargeRecordsByIds(Long[] ids);

    /**
     * 删除用户充值记录信息
     *
     * @param id 用户充值记录主键
     * @return 结果
     */
    public int deleteNgUserRechargeRecordsById(Long id);
}
