package com.fastbee.rechargecard.service;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.fastbee.rechargecard.domain.NgWaterPumpUsageRecords;
import com.fastbee.rechargecard.domain.dto.NgWaterPumpUsageRecordsDto;

/**
 * 水泵设备使用记录Service接口
 * 
 * @author kerwincui
 * @date 2024-12-19
 */
public interface INgWaterPumpUsageRecordsService 
{
    /**
     * 灌溉控制器瞬时流量图表
     * @param deviceNumber
     * @return
     */
    public Map<String,Object> selectNgWaterPumpUsageRecordsInsFlowCharts(String deviceNumber, Date startTime,Date endTime);

    /**
     * 可根据时间段 查询水泵设备使用记录列表
     *
     * @param ngWaterPumpUsageRecords 水泵设备使用记录
     * @return 水泵设备使用记录集合
     */
    public List<NgWaterPumpUsageRecords> selectNgWaterPumpUsageRecordsListByTime(NgWaterPumpUsageRecordsDto ngWaterPumpUsageRecords);
    /**
     * 查询水泵设备使用记录
     * 
     * @param id 水泵设备使用记录主键
     * @return 水泵设备使用记录
     */
    public NgWaterPumpUsageRecords selectNgWaterPumpUsageRecordsById(Long id);
    /**
     * 查询水泵设备最细实时数据
     * @param deviceNumber 水泵设备编码
     * @return 水泵设备使用记录
     */
    public HashMap<String,Object> selectNgWaterPumpUsageDeviceNumber(String deviceNumber);
    /**
     * 查询水泵设备使用记录列表
     * 
     * @param ngWaterPumpUsageRecords 水泵设备使用记录
     * @return 水泵设备使用记录集合
     */
    public List<NgWaterPumpUsageRecords> selectNgWaterPumpUsageRecordsList(NgWaterPumpUsageRecords ngWaterPumpUsageRecords);

    /**
     * 新增水泵设备使用记录
     * 
     * @param ngWaterPumpUsageRecords 水泵设备使用记录
     * @return 结果
     */
    public int insertNgWaterPumpUsageRecords(NgWaterPumpUsageRecords ngWaterPumpUsageRecords);

    /**
     * 修改水泵设备使用记录
     * 
     * @param ngWaterPumpUsageRecords 水泵设备使用记录
     * @return 结果
     */
    public int updateNgWaterPumpUsageRecords(NgWaterPumpUsageRecords ngWaterPumpUsageRecords);

    /**
     * 批量删除水泵设备使用记录
     * 
     * @param ids 需要删除的水泵设备使用记录主键集合
     * @return 结果
     */
    public int deleteNgWaterPumpUsageRecordsByIds(Long[] ids);

    /**
     * 删除水泵设备使用记录信息
     * 
     * @param id 水泵设备使用记录主键
     * @return 结果
     */
    public int deleteNgWaterPumpUsageRecordsById(Long id);
}
