package com.fastbee.rechargecard.service;

import java.util.List;
import com.fastbee.rechargecard.domain.UserConsumptionDetails;
import com.fastbee.rechargecard.domain.dto.UserConsumptionDetailsDto;
import com.fastbee.rechargecard.domain.dto.WeChatRecharge;
import org.springframework.stereotype.Service;

/**
 * 用户充值卡账单明细记录Service接口
 *
 * @author kerwincui
 * @date 2024-12-18
 */

public interface IUserConsumptionDetailsService
{
    /**
     * 根据卡号查询展示的账单信息
     * @param
     * @return
     */
    public List<UserConsumptionDetailsDto> selectUserConsumptionDetailsListByCardNumber(UserConsumptionDetails userConsumptionDetails);
    /**
     * 根据卡号查询用户充值卡账单明细记录列表
     *
     * @param cardNumber 卡号
     * @return 用户充值卡账单明细记录
     */
    public List<UserConsumptionDetails> selectUserConsumptionDetailsListById(String cardNumber);

    /**
     * 查询用户充值卡账单明细记录
     *
     * @param id 用户充值卡账单明细记录主键
     * @return 用户充值卡账单明细记录
     */
    public UserConsumptionDetails selectUserConsumptionDetailsById(Long id);

    /**
     * 查询用户充值卡账单明细记录列表
     *
     * @param userConsumptionDetails 用户充值卡账单明细记录
     * @return 用户充值卡账单明细记录集合
     */
    public List<UserConsumptionDetails> selectUserConsumptionDetailsList(UserConsumptionDetails userConsumptionDetails);

    /**
     * 查询用户充值卡账单明细记录展示列表
     *
     * @param userConsumptionDetails 用户充值卡账单明细记录
     * @return 用户充值卡账单明细记录集合
     */
    public List<UserConsumptionDetailsDto> selectUserConsumptionDetailsShowList(UserConsumptionDetails userConsumptionDetails);

    /**
     * 新增用户充值卡账单明细记录
     *
     * @param userConsumptionDetails 用户充值卡账单明细记录
     * @return 结果
     */
    public int insertUserConsumptionDetails(UserConsumptionDetails userConsumptionDetails);

    /**
     * 修改用户充值卡账单明细记录
     *
     * @param userConsumptionDetails 用户充值卡账单明细记录
     * @return 结果
     */
    public int updateUserConsumptionDetails(UserConsumptionDetails userConsumptionDetails);

    /**
     * 批量删除用户充值卡账单明细记录
     *
     * @param ids 需要删除的用户充值卡账单明细记录主键集合
     * @return 结果
     */
    public int deleteUserConsumptionDetailsByIds(Long[] ids);

    /**
     * 删除用户充值卡账单明细记录信息
     *
     * @param id 用户充值卡账单明细记录主键
     * @return 结果
     */
    public int deleteUserConsumptionDetailsById(Long id);

    /**
     * 插入微信支付用户充值卡账单明细记录
     */
    public int insertUserConsumptionDetailsWechat(WeChatRecharge recharge);
}
